/**
 * License Agreement.
 *
 * Ajax4jsf 1.1 - Natural Ajax for Java Server Faces (JSF)
 *
 * Copyright (C) 2007 Exadel, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */

package org.richfaces.renderkit.html;


// 
// Imports
//
import java.util.Iterator;
import java.util.Collection;
import java.io.IOException;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import org.ajax4jsf.framework.renderer.ComponentsVariableResolver;
import org.ajax4jsf.framework.renderer.ComponentVariables;
import org.ajax4jsf.framework.resource.InternetResource;
import org.ajax4jsf.framework.resource.InternetResource;
//
//
//


import org.richfaces.renderkit.html.DataScrollerRenderer;



/**
 * Renderer for component class org.richfaces.renderkit.html.DatascrollerTemplate
 */
public class DatascrollerTemplate extends DataScrollerRenderer {

	public DatascrollerTemplate () {
		super();
	}

	// 
	// Declarations
	//
	private final InternetResource[] styles = {
						getResource("css/datascroller.xcss")
	};

private InternetResource[] stylesAll = null;

protected InternetResource[] getStyles() {
	synchronized (this) {
		if (stylesAll == null) {
			InternetResource[] rsrcs = super.getStyles();
			boolean ignoreSuper = rsrcs == null || rsrcs.length == 0;
			boolean ignoreThis = styles == null || styles.length == 0;
			
			if (ignoreSuper) {
				if (ignoreThis) {
					stylesAll = new InternetResource[0];	
				} else {
					stylesAll = styles;
				}
			} else {
				if (ignoreThis) {
					stylesAll = rsrcs;
				} else {
					java.util.Set rsrcsSet = new java.util.LinkedHashSet();

					for (int i = 0; i < rsrcs.length; i++ ) {
						rsrcsSet.add(rsrcs[i]);
					}

					for (int i = 0; i < styles.length; i++ ) {
						rsrcsSet.add(styles[i]);
					}

					stylesAll = (InternetResource[]) rsrcsSet.toArray(new InternetResource[rsrcsSet.size()]);
				}
			}
		}
	}
	
	return stylesAll;
}
	private final InternetResource[] scripts = {
						new org.ajax4jsf.framework.resource.PrototypeScript()
						,
				new org.ajax4jsf.framework.ajax.AjaxScript()
	};

private InternetResource[] scriptsAll = null;

protected InternetResource[] getScripts() {
	synchronized (this) {
		if (scriptsAll == null) {
			InternetResource[] rsrcs = super.getScripts();
			boolean ignoreSuper = rsrcs == null || rsrcs.length == 0;
			boolean ignoreThis = scripts == null || scripts.length == 0;
			
			if (ignoreSuper) {
				if (ignoreThis) {
					scriptsAll = new InternetResource[0];	
				} else {
					scriptsAll = scripts;
				}
			} else {
				if (ignoreThis) {
					scriptsAll = rsrcs;
				} else {
					java.util.Set rsrcsSet = new java.util.LinkedHashSet();

					for (int i = 0; i < rsrcs.length; i++ ) {
						rsrcsSet.add(rsrcs[i]);
					}

					for (int i = 0; i < scripts.length; i++ ) {
						rsrcsSet.add(scripts[i]);
					}

					scriptsAll = (InternetResource[]) rsrcsSet.toArray(new InternetResource[rsrcsSet.size()]);
				}
			}
		}
	}
	
	return scriptsAll;
}
	// 
	// 
	//


	private String convertToString(Object obj ) {
		return ( obj == null ? "" : obj.toString() );
	}
	private String convertToString(boolean b ) {
		return String.valueOf(b);
	}
	private String convertToString(int b ) {
		return b!=Integer.MIN_VALUE?String.valueOf(b):"";
	}
	private String convertToString(long b ) {
		return b!=Long.MIN_VALUE?String.valueOf(b):"";
	}
	
	/**
	 * Get base component class, targetted for this renderer. Used for check arguments in decode/encode.
	 * @return
	 */
	protected Class getComponentClass() {
		return org.richfaces.component.UIDatascroller.class;
	}

	
	public void doEncodeBegin(ResponseWriter writer, FacesContext context, UIComponent component ) throws IOException {
		ComponentVariables variables = ComponentsVariableResolver.getVariables(this, component);
		doEncodeBegin(writer, context, (org.richfaces.component.UIDatascroller)component, variables );
	}		

	public void doEncodeBegin(ResponseWriter writer, FacesContext context, org.richfaces.component.UIDatascroller component, ComponentVariables variables ) throws IOException {
	    java.lang.String clientId = component.getClientId(context);

		org.richfaces.component.util.FormUtil.throwEnclFormReqExceptionIfNeed(context,component);		                                               
    
writer.startElement("div", component);
			getUtils().writeAttribute(writer, "class", "dr-dscr rich-datascr " + convertToString(component.getAttributes().get("styleClass")) );
						getUtils().writeAttribute(writer, "id", clientId );
			//
// pass thru attributes
//
getUtils().encodeAttributesFromArray(context,component,new String[] {
    "class" ,
	    "dir" ,
	    "lang" ,
	    "onclick" ,
	    "ondblclick" ,
	    "onkeydown" ,
	    "onkeypress" ,
	    "onkeyup" ,
	    "onmousedown" ,
	    "onmousemove" ,
	    "onmouseout" ,
	    "onmouseover" ,
	    "onmouseup" ,
	    "style" ,
	    "title" ,
	    "xml:lang" });
//
//
//

writer.startElement("table", component);
			getUtils().writeAttribute(writer, "align", component.getAttributes().get("align") );
						getUtils().writeAttribute(writer, "border", "0" );
						getUtils().writeAttribute(writer, "cellpadding", "0" );
						getUtils().writeAttribute(writer, "cellspacing", "1" );
						getUtils().writeAttribute(writer, "class", "dr-dscr-t rich-dtascroller-table " + convertToString(component.getAttributes().get("tableStyleClass")) );
						getUtils().writeAttribute(writer, "id", convertToString(clientId) + "_table" );
			
writer.startElement("tbody", component);

writer.startElement("tr", component);


                String facet;                         
                org.richfaces.renderkit.html.ControlsState controlsState = getControlsState(context, component, 
                		(javax.faces.component.UIData) variables.getVariable("dataTable"));     

                
                if (controlsState.isFirstRendered()){                       
                if (controlsState.isFirstEnabled()){
                  variables.setVariable("color", "");                  
                  variables.setVariable("onclick", getOnClick(context,component,component.FIRST_FACET_NAME));                        
                  variables.setVariable("facet", component.FIRST_FACET_NAME);                  
                  facet=component.FIRST_FACET_NAME;
                }else{
                  variables.setVariable("color", "color: " + org.ajax4jsf.framework.skin.SkinFactory.getInstance().getSkin(context).getParameter(context, "panelBorderColor", "gray")+ ";");  
                  variables.setVariable("onclick", "");                                            
                  variables.setVariable("facet", component.FIRST_DISABLED_FACET_NAME);                                    
                  facet=component.FIRST_DISABLED_FACET_NAME;
                };                
                if(component.getFacet(facet)!=null ) {
                
writer.startElement("td", component);
			getUtils().writeAttribute(writer, "align", "center" );
						getUtils().writeAttribute(writer, "class", "dr-dscr-button rich-datascr-button" );
						getUtils().writeAttribute(writer, "onclick", variables.getVariable("onclick") );
						getUtils().writeAttribute(writer, "style", variables.getVariable("color") );
			

					     if(component.FIRST_FACET_NAME.equals(facet)){
				       
UIComponent indexChildren_8 = component.getFacet("first");
if (null != indexChildren_8 && indexChildren_8 .isRendered()) {
	renderChild(context, indexChildren_8);
}


					     }else{
				       
UIComponent indexChildren_9 = component.getFacet("first_disabled");
if (null != indexChildren_9 && indexChildren_9 .isRendered()) {
	renderChild(context, indexChildren_9);
}


					     }
				       
writer.endElement("td");

                }else{
                
writer.startElement("td", component);
			getUtils().writeAttribute(writer, "align", "center" );
						getUtils().writeAttribute(writer, "class", "dr-dscr-button rich-datascr-button" );
						getUtils().writeAttribute(writer, "onclick", variables.getVariable("onclick") );
						getUtils().writeAttribute(writer, "style", variables.getVariable("color") );
			
                    
                      writer.write("&#171;&#171;");                      
                    
writer.endElement("td");

                }
                }
                
                
                if (controlsState.isFastRewindRendered()){                       
                if (controlsState.isFastRewindEnabled()){
                  variables.setVariable("color", "");    
                  variables.setVariable("onclick", getOnClick(context,component,component.FAST_REWIND_FACET_NAME));                                                                
                  variables.setVariable("facet", component.FAST_REWIND_FACET_NAME);                                    
                  facet=component.FAST_REWIND_FACET_NAME;                  
                }else{
                    variables.setVariable("color", "color: " + org.ajax4jsf.framework.skin.SkinFactory.getInstance().getSkin(context).getParameter(context, "panelBorderColor", "gray")+ ";");  
                    variables.setVariable("onclick", "");                                                                                    
                    variables.setVariable("facet", component.FAST_REWIND_DISABLED_FACET_NAME);                                    
                    facet=component.FAST_REWIND_DISABLED_FACET_NAME;                                      
                }
                if(component.getFacet(facet)!=null ) {
                
writer.startElement("td", component);
			getUtils().writeAttribute(writer, "align", "center" );
						getUtils().writeAttribute(writer, "class", "dr-dscr-button rich-datascr-button" );
						getUtils().writeAttribute(writer, "onclick", variables.getVariable("onclick") );
						getUtils().writeAttribute(writer, "style", variables.getVariable("color") );
			

					     if(component.FAST_REWIND_FACET_NAME.equals(facet)){
				       
UIComponent indexChildren_10 = component.getFacet("fastrewind");
if (null != indexChildren_10 && indexChildren_10 .isRendered()) {
	renderChild(context, indexChildren_10);
}


					     }else{
				       
UIComponent indexChildren_11 = component.getFacet("fastrewind_disabled");
if (null != indexChildren_11 && indexChildren_11 .isRendered()) {
	renderChild(context, indexChildren_11);
}


					     }
				       
writer.endElement("td");

                }else{
                
writer.startElement("td", component);
			getUtils().writeAttribute(writer, "align", "center" );
						getUtils().writeAttribute(writer, "class", "dr-dscr-button rich-datascr-button" );
						getUtils().writeAttribute(writer, "onclick", variables.getVariable("onclick") );
						getUtils().writeAttribute(writer, "style", variables.getVariable("color") );
			
                    
                      writer.write("&#171;");                      
                    
writer.endElement("td");

                }
                }                
                
                                                                                 
                if (controlsState.isPreviousRendered()){                       
                if (controlsState.isPreviousEnabled()){                     
                  variables.setVariable("color", "");    
                  variables.setVariable("onclick", getOnClick(context,component,component.PREVIOUS_FACET_NAME));                                                                
                  variables.setVariable("facet", component.PREVIOUS_FACET_NAME);                                    
                  facet=component.PREVIOUS_FACET_NAME;                                    
                }else{
                  variables.setVariable("onclick", "");                                                                                    
                  variables.setVariable("color", "color: " + org.ajax4jsf.framework.skin.SkinFactory.getInstance().getSkin(context).getParameter(context, "panelBorderColor", "gray")+ ";");  
                  variables.setVariable("facet", component.PREVIOUS_DISABLED_FACET_NAME);                                    
                  facet=component.PREVIOUS_DISABLED_FACET_NAME;                  
                }
                if(component.getFacet(facet)!=null ) {
                
writer.startElement("td", component);
			getUtils().writeAttribute(writer, "align", "center" );
						getUtils().writeAttribute(writer, "class", "dr-dscr-button rich-datascr-button" );
						getUtils().writeAttribute(writer, "onclick", variables.getVariable("onclick") );
						getUtils().writeAttribute(writer, "style", variables.getVariable("color") );
			

					     if(component.PREVIOUS_FACET_NAME.equals(facet)){
				       
UIComponent indexChildren_12 = component.getFacet("previous");
if (null != indexChildren_12 && indexChildren_12 .isRendered()) {
	renderChild(context, indexChildren_12);
}


					     }else{
				       
UIComponent indexChildren_13 = component.getFacet("previous_disabled");
if (null != indexChildren_13 && indexChildren_13 .isRendered()) {
	renderChild(context, indexChildren_13);
}


					     }
				       
writer.endElement("td");

                }else{
                
writer.startElement("td", component);
			getUtils().writeAttribute(writer, "align", "center" );
						getUtils().writeAttribute(writer, "class", "dr-dscr-button rich-datascr-button" );
						getUtils().writeAttribute(writer, "onclick", variables.getVariable("onclick") );
						getUtils().writeAttribute(writer, "style", variables.getVariable("color") );
			
writer.endElement("td");

                 }
                 }                
                 
renderPager(context, component);


                if (controlsState.isNextRendered()){                       
                if (controlsState.isNextEnabled()){                                                                               
                  variables.setVariable("onclick", getOnClick(context,component,component.NEXT_FACET_NAME));                                        
                  variables.setVariable("color", "");  
                  variables.setVariable("facet", component.NEXT_FACET_NAME);                                    
                  facet=component.NEXT_FACET_NAME;                                                      
                }else{
                  variables.setVariable("onclick", "");  
                  variables.setVariable("color", "color: " + org.ajax4jsf.framework.skin.SkinFactory.getInstance().getSkin(context).getParameter(context, "panelBorderColor", "gray")+ ";");  
                  variables.setVariable("facet", component.NEXT_DISABLED_FACET_NAME);                                    
                  facet=component.NEXT_DISABLED_FACET_NAME;                                    
                }
                if(component.getFacet(facet)!=null ) {
                
writer.startElement("td", component);
			getUtils().writeAttribute(writer, "align", "center" );
						getUtils().writeAttribute(writer, "class", "dr-dscr-button rich-datascr-button" );
						getUtils().writeAttribute(writer, "onclick", variables.getVariable("onclick") );
						getUtils().writeAttribute(writer, "style", variables.getVariable("color") );
			

					     if(component.NEXT_FACET_NAME.equals(facet)){
				       
UIComponent indexChildren_14 = component.getFacet("next");
if (null != indexChildren_14 && indexChildren_14 .isRendered()) {
	renderChild(context, indexChildren_14);
}


					     }else{
				       
UIComponent indexChildren_15 = component.getFacet("next_disabled");
if (null != indexChildren_15 && indexChildren_15 .isRendered()) {
	renderChild(context, indexChildren_15);
}


					     }
				       
writer.endElement("td");

                }else{
                
writer.startElement("td", component);
			getUtils().writeAttribute(writer, "align", "center" );
						getUtils().writeAttribute(writer, "class", "dr-dscr-button rich-datascr-button" );
						getUtils().writeAttribute(writer, "onclick", variables.getVariable("onclick") );
						getUtils().writeAttribute(writer, "style", variables.getVariable("color") );
			
writer.endElement("td");

                }
                }                
                

                if (controlsState.isFastForwardRendered()){                       
                if (controlsState.isFastForwardEnabled()){                                                                                                                       
                  variables.setVariable("onclick", getOnClick(context,component,component.FAST_FORWARD_FACET_NAME));
                  variables.setVariable("color", "");  
                  variables.setVariable("facet", component.FAST_FORWARD_FACET_NAME);                                    
                  facet=component.FAST_FORWARD_FACET_NAME;                                                      
                }
                else{
                  variables.setVariable("onclick", "");  
                  variables.setVariable("color", "color: " + org.ajax4jsf.framework.skin.SkinFactory.getInstance().getSkin(context).getParameter(context, "panelBorderColor", "gray")+ ";");  
                  variables.setVariable("facet", component.FAST_FORWARD_DISABLED_FACET_NAME);                                    
                  facet=component.FAST_FORWARD_DISABLED_FACET_NAME;                                                      
                }
                if(component.getFacet(facet)!=null ) {
                
writer.startElement("td", component);
			getUtils().writeAttribute(writer, "align", "center" );
						getUtils().writeAttribute(writer, "class", "dr-dscr-button rich-datascr-button" );
						getUtils().writeAttribute(writer, "onclick", variables.getVariable("onclick") );
						getUtils().writeAttribute(writer, "style", variables.getVariable("color") );
			

					     if(component.FAST_FORWARD_FACET_NAME.equals(facet)){
				       
UIComponent indexChildren_16 = component.getFacet("fastforward");
if (null != indexChildren_16 && indexChildren_16 .isRendered()) {
	renderChild(context, indexChildren_16);
}


					     }else{
				       
UIComponent indexChildren_17 = component.getFacet("fastforward_disabled");
if (null != indexChildren_17 && indexChildren_17 .isRendered()) {
	renderChild(context, indexChildren_17);
}


					     }
				       
writer.endElement("td");

                }else{
                
writer.startElement("td", component);
			getUtils().writeAttribute(writer, "align", "center" );
						getUtils().writeAttribute(writer, "class", "dr-dscr-button rich-datascr-button" );
						getUtils().writeAttribute(writer, "onclick", variables.getVariable("onclick") );
						getUtils().writeAttribute(writer, "style", variables.getVariable("color") );
			
                    
                      writer.write("&#187;");                      
                    
writer.endElement("td");

                }
                }                
                

                if (controlsState.isLastRendered()){                       
                if (controlsState.isLastEnabled()){                                                                                                                                                             
                  variables.setVariable("onclick", getOnClick(context,component,component.LAST_FACET_NAME));                                        
                  variables.setVariable("color", "");  
                  variables.setVariable("facet", component.LAST_FACET_NAME);                                    
                  facet=component.LAST_FACET_NAME;                                                                                          
                }else{
                  variables.setVariable("onclick", "");                                                            
                  variables.setVariable("color", "color: " + org.ajax4jsf.framework.skin.SkinFactory.getInstance().getSkin(context).getParameter(context, "panelBorderColor", "gray")+ ";");  
                  variables.setVariable("facet", component.LAST_DISABLED_FACET_NAME);                                    
                  facet=component.LAST_DISABLED_FACET_NAME;                                                                        
                }
                if(component.getFacet(facet)!=null ) {
                
writer.startElement("td", component);
			getUtils().writeAttribute(writer, "align", "center" );
						getUtils().writeAttribute(writer, "class", "dr-dscr-button rich-datascr-button" );
						getUtils().writeAttribute(writer, "onclick", variables.getVariable("onclick") );
						getUtils().writeAttribute(writer, "style", variables.getVariable("color") );
			

					     if(component.LAST_FACET_NAME.equals(facet)){
				       
UIComponent indexChildren_18 = component.getFacet("last");
if (null != indexChildren_18 && indexChildren_18 .isRendered()) {
	renderChild(context, indexChildren_18);
}


					     }else{
				       
UIComponent indexChildren_19 = component.getFacet("last_disabled");
if (null != indexChildren_19 && indexChildren_19 .isRendered()) {
	renderChild(context, indexChildren_19);
}


					     }
				       
writer.endElement("td");

                }else{
                
writer.startElement("td", component);
			getUtils().writeAttribute(writer, "align", "center" );
						getUtils().writeAttribute(writer, "class", "dr-dscr-button rich-datascr-button" );
						getUtils().writeAttribute(writer, "onclick", variables.getVariable("onclick") );
						getUtils().writeAttribute(writer, "style", variables.getVariable("color") );
			
                    
                          writer.write("&#187;&#187;");                      
                        
writer.endElement("td");

                }
                }                
                
writer.endElement("tr");

	}		
	
	public void doEncodeEnd(ResponseWriter writer, FacesContext context, org.richfaces.component.UIDatascroller component, ComponentVariables variables) throws IOException {
	  
writer.endElement("tbody");
writer.endElement("table");
writer.endElement("div");

	}		
	
	public void doEncodeEnd(ResponseWriter writer, FacesContext context, UIComponent component) throws IOException {
		ComponentVariables variables = ComponentsVariableResolver.getVariables(this, component);
		doEncodeEnd(writer, context, (org.richfaces.component.UIDatascroller)component, variables );

		ComponentsVariableResolver.removeVariables(this, component);
	}		
	

}
