package org.richfaces.renderkit.html.images;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.io.Serializable;
import java.util.Date;

import javax.faces.context.FacesContext;

import org.ajax4jsf.framework.resource.GifRenderer;
import org.ajax4jsf.framework.resource.InternetResourceBuilder;
import org.ajax4jsf.framework.resource.Java2Dresource;
import org.ajax4jsf.framework.resource.ResourceContext;
import org.ajax4jsf.framework.skin.Skin;
import org.ajax4jsf.framework.skin.SkinFactory;
import org.ajax4jsf.framework.util.HtmlColor;

public class MenuNodeImage extends Java2Dresource {

    private static final Dimension dimensions = new Dimension(16, 32);
    
    protected Dimension getDimensions(ResourceContext resourceContext) {
        return dimensions;
    }
    public Dimension getDimensions(FacesContext facesContext, Object data) {
        return dimensions;
    }

    public MenuNodeImage() {
        setRenderer(new GifRenderer());
        setLastModified(new Date(InternetResourceBuilder.getInstance().getStartTime()));
    }

    protected Object getDataToStore(FacesContext context, Object data) {
        
        MenuNodeImageData mnd = new MenuNodeImageData();
        Skin skin = SkinFactory.getInstance().getSkin(context);
        
        String tmp;
        int intValue;
        
        tmp = (String) skin.getParameter(context,"generalTextColor");
        if (tmp == null) tmp = "#4A75B5";
        intValue = HtmlColor.decode(tmp).getRGB();
        mnd.setGeneralColor(new Integer(intValue));
        
        tmp = (String) skin.getParameter(context,"tabDisabledTextColor");
        if (tmp == null) tmp = "#6A92CF";
        intValue = HtmlColor.decode(tmp).getRGB();
        mnd.setDisabledColor(new Integer(intValue));
        
        return mnd;
    }
    
    protected void paint(ResourceContext context, Graphics2D g2d) {
        MenuNodeImageData mnd = (MenuNodeImageData) restoreData(context);
        g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        
        int x0 = 6;
        int y0 = 8;

        g2d.setColor(new Color(mnd.getGeneralColor().intValue()));
        g2d.drawLine(x0, y0 - 3, x0, y0 +3);
        g2d.drawLine(x0 + 1, y0 - 2, x0 + 1, y0 + 2);
        g2d.drawLine(x0 + 2, y0 - 1, x0 + 2, y0 + 1);
        g2d.drawLine(x0 + 3, y0, x0 + 3, y0);

        int y1 = 24;

        g2d.setColor(new Color(mnd.getDisabledColor().intValue()));
        g2d.drawLine(x0, y1 - 3, x0, y1 +3);
        g2d.drawLine(x0 + 1, y1 - 2, x0 + 1, y1 + 2);
        g2d.drawLine(x0 + 2, y1 - 1, x0 + 2, y1 + 1);
        g2d.drawLine(x0 + 3, y1, x0 + 3, y1);
    }
    
    
    protected static class MenuNodeImageData implements Serializable{
        private static final long serialVersionUID = 1732700513743861251L;
        
        private Integer disabledColor;
        private Integer generalColor;
        
        public Integer getDisabledColor() {
            return disabledColor;
        }
        public void setDisabledColor(Integer disabledColor) {
            this.disabledColor = disabledColor;
        }
        public Integer getGeneralColor() {
            return generalColor;
        }
        public void setGeneralColor(Integer generalColor) {
            this.generalColor = generalColor;
        }
    }
}
